from string import ascii_uppercase

from ... import BaseProvider


class Provider(BaseProvider):
    """
    Provider for Philippine automotive license plates

    Vehicle registration in the Philippines has many controversies and is full of quirks. On top of that, some terms are
    highly subject to interpretation or to varying definitions when applied colloquially, e.g. "motor" usually refers to
    either a machine's motor or a motorcycle, "vehicles" usually means cars, SUVs, vans, and trucks but not motorcycles.
    All of these, interestingly, affect how and what kind of license plates are issued. For the purposes of this
    provider, the following pointers apply:
    - High ranking government officials are entitled to use low numbered protocol license plates.
    - Motorcycles and any improvised vehicle with a motorcycle as its base are issued motorcycle license plates.
    - Cars, SUVs, vans, trucks, and other 4-wheeled civilian vehicles are considered automobiles.
    - Invoking `license_plate()` will never generate protocol plates, because those are plates for specific use cases.

    Sources:
    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_the_Philippines
    """

    protocol_licenses = tuple(str(x) for x in range(1, 18) if x != 15)
    motorcycle_license_formats = (
        '??####',     # 1981 series
        '??#####',    # 2014 series
    )
    automobile_license_formats = (
        '???###',    # 1981 series
        '???####',   # 2014 series
    )
    license_formats = motorcycle_license_formats + automobile_license_formats

    def _license_plate(self, license_format):
        return self.bothify(self.random_element(license_format), ascii_uppercase)

    def protocol_license_plate(self):
        return self.random_element(self.protocol_licenses)

    def motorcycle_license_plate(self):
        return self._license_plate(self.motorcycle_license_formats)

    def automobile_license_plate(self):
        return self._license_plate(self.automobile_license_formats)

    def license_plate(self):
        return self._license_plate(self.license_formats)



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{prefix}} {{last_name}}',
        '{{first_name}} {{last_name}}-{{last_name}}',
        '{{first_name}}-{{first_name}} {{last_name}}',
    )

    first_names = (
        'Adriana', 'Afonso', 'Alex', 'Alexandra', 'Alexandre', 'Alice',
        'Alícia', 'Amélia', 'Ana', 'Andreia', 'André', 'Anita', 'António',
        'Ariana', 'Artur', 'Beatriz', 'Benedita', 'Benjamim', 'Bernardo',
        'Bianca', 'Brian', 'Bruna', 'Bruno', 'Bryan', 'Bárbara', 'Caetana',
        'Camila', 'Carlos', 'Carlota', 'Carminho', 'Carolina', 'Catarina',
        'Clara', 'Cláudio', 'Constança', 'Cristiano', 'César', 'Daniel',
        'Daniela', 'David', 'Denis', 'Diana', 'Diego', 'Dinis', 'Diogo',
        'Duarte', 'Débora', 'Edgar', 'Eduarda', 'Eduardo', 'Ema', 'Emanuel',
        'Emma', 'Emília', 'Enzo', 'Erica', 'Erika', 'Eva', 'Fabiana',
        'Fernando', 'Filipa', 'Filipe', 'Flor', 'Francisca', 'Francisco',
        'Frederico', 'Fábio', 'Gabriel', 'Gabriela', 'Gaspar', 'Gil', 'Gonçalo',
        'Guilherme', 'Gustavo', 'Helena', 'Henrique', 'Hugo', 'Iara', 'Igor',
        'Inês', 'Irina', 'Isaac', 'Isabel', 'Isabela', 'Ivan', 'Ivo', 'Jaime',
        'Joana', 'Joaquim', 'Joel', 'Jorge', 'José', 'João', 'Juliana',
        'Jéssica', 'Júlia', 'Kelly', 'Kevin', 'Kyara', 'Kévim', 'Lara',
        'Larissa', 'Laura', 'Leandro', 'Leonardo', 'Leonor', 'Letícia', 'Lia',
        'Lisandro', 'Lorena', 'Lourenço', 'Luana', 'Luca', 'Lucas', 'Luciana',
        'Luna', 'Luís', 'Luísa', 'Lúcia', 'Madalena', 'Mafalda', 'Manuel',
        'Mara', 'Marco', 'Marcos', 'Margarida', 'Maria', 'Mariana', 'Marta',
        'Martim', 'Mateus', 'Matias', 'Matilde', 'Mauro', 'Melissa', 'Mia',
        'Micael', 'Miguel', 'Miriam', 'Márcio', 'Mário', 'Mélanie', 'Naiara',
        'Nair', 'Nelson', 'Nicole', 'Noa', 'Noah', 'Nuno', 'Nádia', 'Núria',
        'Patrícia', 'Paulo', 'Pedro', 'Petra', 'Pilar', 'Rafael', 'Rafaela',
        'Raquel', 'Renata', 'Renato', 'Ricardo', 'Rita', 'Rodrigo', 'Rui',
        'Rúben', 'Salomé', 'Salvador', 'Samuel', 'Sandro', 'Santiago', 'Sara',
        'Sebastião', 'Simão', 'Sofia', 'Soraia', 'Sérgio', 'Tatiana', 'Teresa',
        'Tiago', 'Tomás', 'Tomé', 'Valentim', 'Valentina', 'Vasco', 'Vera',
        'Vicente', 'Victória', 'Violeta', 'Vitória', 'Vítor', 'William',
        'Wilson', 'Xavier', 'Yara', 'Yasmin', 'Álvaro', 'Ângela', 'Ângelo',
        'Érica', 'Íris',
    )

    last_names = (
        'Abreu', 'Almeida', 'Alves', 'Amaral', 'Amorim', 'Andrade', 'Anjos',
        'Antunes', 'Araújo', 'Assunção', 'Azevedo', 'Baptista', 'Barbosa',
        'Barros', 'Batista', 'Borges', 'Branco', 'Brito', 'Campos', 'Cardoso',
        'Carneiro', 'Carvalho', 'Castro', 'Coelho', 'Correia', 'Costa', 'Cruz',
        'Cunha', 'Domingues', 'Esteves', 'Faria', 'Fernandes', 'Ferreira',
        'Figueiredo', 'Fonseca', 'Freitas', 'Garcia', 'Gaspar', 'Gomes',
        'Gonçalves', 'Guerreiro', 'Henriques', 'Jesus', 'Leal', 'Leite', 'Lima',
        'Lopes', 'Loureiro', 'Lourenço', 'Macedo', 'Machado', 'Magalhães',
        'Maia', 'Marques', 'Martins', 'Matias', 'Matos', 'Melo', 'Mendes',
        'Miranda', 'Monteiro', 'Morais', 'Moreira', 'Mota', 'Moura',
        'Nascimento', 'Neto', 'Neves', 'Nogueira', 'Nunes', 'Oliveira',
        'Pacheco', 'Paiva', 'Pereira', 'Pinheiro', 'Pinho', 'Pinto', 'Pires',
        'Ramos', 'Reis', 'Ribeiro', 'Rocha', 'Rodrigues', 'Santos', 'Silva',
        'Simões', 'Soares', 'Sousa', 'Sá', 'Tavares', 'Teixeira', 'Torres',
        'Valente', 'Vaz', 'Vicente', 'Vieira',
    )

    prefixes = ('de', 'da', 'do')

    def prefix(self):
        return self.random_element(self.prefixes)



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = [
        '{{last_name}} {{company_suffix}}',
        '{{last_name}} {{company_suffix}}',
        '{{last_name}} {{company_suffix}}',
        '{{last_name}}-{{last_name}} {{company_suffix}}',
        '{{last_name}}, {{last_name}} og {{last_name}}',
        '{{last_name}}-{{last_name}}',
    ]

    company_suffixes = [
        'Gruppen', 'AS', 'ASA', 'BA', 'RFH', 'og Sønner', '& co.',
    ]



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        '{{last_name}} {{company_suffix}}',
        '{{last_name}} {{last_name}} {{company_suffix}}',
        '{{last_name}} {{last_name}} {{company_suffix}}',
        '{{last_name}}',
    )

    company_suffixes = (
        'As Oy', 'Tmi', 'Oy', 'Oyj', 'Ky', 'Osk', 'ry',
    )

    def company_business_id(self):
        """
        Returns Finnish company Business Identity Code (y-tunnus).
        Format is 8 digits - e.g. FI99999999,[8] last digit is a check
        digit utilizing MOD 11-2. The first digit is zero for some old
        organizations. This function provides current codes starting with
        non-zero.
        """
        def calculate_checksum(number):
            """Calculate the checksum using mod 11,2 method"""
            factors = [7, 9, 10, 5, 8, 4, 2]
            sum_ = 0
            for x, y in zip(number, factors):
                sum_ = sum_ + int(x) * y
            if sum_ % 11 == 0:
                return '0'
            else:
                return str(11 - sum_ % 11)

        first_digit = str(self.random_digit_not_null())
        body = first_digit + self.bothify('######')
        cs = calculate_checksum(body)
        return body + '-' + str(cs)

    def company_vat(self):
        """
        Returns Finnish VAT identification number (Arvonlisaveronumero).
        This can be calculated from company business identity code by
        adding prefix "FI" and removing dash before checksum.
        """
        def convert_to_vat(business_id):
            """
            Convert business id to VATIN
            """
            return 'FI' + business_id.replace('-', '')

        return convert_to_vat(self.company_business_id())



import csv
import hashlib
import io
import string
import tarfile
import uuid
import zipfile

from .. import BaseProvider

localized = True

csv.register_dialect('faker-csv', csv.excel, quoting=csv.QUOTE_ALL)


class Provider(BaseProvider):

    def boolean(self, chance_of_getting_true=50):
        """Generate a random boolean value based on ``chance_of_getting_true``.

        :sample size=10: chance_of_getting_true=25
        :sample size=10: chance_of_getting_true=50
        :sample size=10: chance_of_getting_true=75
        """
        return self.generator.random.randint(1, 100) <= chance_of_getting_true

    def null_boolean(self):
        """Generate ``None``, ``True``, or ``False``, each with equal probability.

        :sample size=15:
        """
        return {
            0: None,
            1: True,
            -1: False,
        }[self.generator.random.randint(-1, 1)]

    def binary(self, length=(1 * 1024 * 1024)):
        """Generate a random binary blob of ``length`` bytes.

        :sample: length=64
        """
        blob = [self.generator.random.randrange(256) for _ in range(length)]
        return bytes(blob)

    def md5(self, raw_output=False):
        """Generate a random MD5 hash.

        If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the MD5 hash
        will be returned. If ``True``, a ``bytes`` object representation will be returned instead.

        :sample: raw_output=False
        :sample: raw_output=True
        """
        res = hashlib.md5(str(self.generator.random.random()).encode())
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def sha1(self, raw_output=False):
        """Generate a random SHA1 hash.

        If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the SHA1 hash
        will be returned. If ``True``, a ``bytes`` object representation will be returned instead.

        :sample: raw_output=False
        :sample: raw_output=True
        """
        res = hashlib.sha1(str(self.generator.random.random()).encode())
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def sha256(self, raw_output=False):
        """Generate a random SHA256 hash.

        If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the SHA56 hash
        will be returned. If ``True``, a ``bytes`` object representation will be returned instead.

        :sample: raw_output=False
        :sample: raw_output=True
        """
        res = hashlib.sha256(
            str(self.generator.random.random()).encode())
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def uuid4(self, cast_to=str):
        """Generate a random UUID4 object and cast it to another type if specified using a callable ``cast_to``.

        By default, ``cast_to`` is set to ``str``.

        May be called with ``cast_to=None`` to return a full-fledged ``UUID``.

        :sample:
        :sample: cast_to=None
        """
        # Based on http://stackoverflow.com/q/41186818
        generated_uuid = uuid.UUID(int=self.generator.random.getrandbits(128), version=4)
        if cast_to is not None:
            generated_uuid = cast_to(generated_uuid)
        return generated_uuid

    def password(
            self,
            length=10,
            special_chars=True,
            digits=True,
            upper_case=True,
            lower_case=True):
        """Generate a random password of the specified ``length``.

        The arguments ``special_chars``, ``digits``, ``upper_case``, and ``lower_case`` control
        what category of characters will appear in the generated password. If set to ``True``
        (default), at least one character from the corresponding category is guaranteed to appear.
        Special characters are characters from ``!@#$%^&*()_+``, digits are characters from
        ``0123456789``, and uppercase and lowercase characters are characters from the ASCII set of
        letters.

        :sample: length=12
        :sample: length=40, special_chars=False, upper_case=False
        """
        choices = ""
        required_tokens = []
        if special_chars:
            required_tokens.append(
                self.generator.random.choice("!@#$%^&*()_+"))
            choices += "!@#$%^&*()_+"
        if digits:
            required_tokens.append(self.generator.random.choice(string.digits))
            choices += string.digits
        if upper_case:
            required_tokens.append(
                self.generator.random.choice(string.ascii_uppercase))
            choices += string.ascii_uppercase
        if lower_case:
            required_tokens.append(
                self.generator.random.choice(string.ascii_lowercase))
            choices += string.ascii_lowercase

        assert len(
            required_tokens) <= length, "Required length is shorter than required characters"

        # Generate a first version of the password
        chars = self.random_choices(choices, length=length)

        # Pick some unique locations
        random_indexes = set()
        while len(random_indexes) < len(required_tokens):
            random_indexes.add(
                self.generator.random.randint(0, len(chars) - 1))

        # Replace them with the required characters
        for i, index in enumerate(random_indexes):
            chars[index] = required_tokens[i]

        return ''.join(chars)

    def zip(self, uncompressed_size=65536, num_files=1, min_file_size=4096, compression=None):
        """Generate a bytes object containing a random valid zip archive file.

        The number and sizes of files contained inside the resulting archive can be controlled
        using the following arguments:

        - ``uncompressed_size`` - the total size of files before compression, 16 KiB by default
        - ``num_files`` - the number of files archived in resulting zip file, 1 by default
        - ``min_file_size`` - the minimum size of each file before compression, 4 KiB by default

        No compression is used by default, but setting ``compression`` to one of the values listed
        below will use the corresponding compression type.

        - ``'bzip2'`` or ``'bz2'`` for BZIP2
        - ``'lzma'`` or ``'xz'`` for LZMA
        - ``'deflate'``, ``'gzip'``, or ``'gz'`` for GZIP

        :sample: uncompressed_size=256, num_files=4, min_file_size=32
        :sample: uncompressed_size=256, num_files=32, min_file_size=4, compression='bz2'
        """
        if any([
            not isinstance(num_files, int) or num_files <= 0,
            not isinstance(min_file_size, int) or min_file_size <= 0,
            not isinstance(uncompressed_size, int) or uncompressed_size <= 0,
        ]):
            raise ValueError(
                '`num_files`, `min_file_size`, and `uncompressed_size` must be positive integers',
            )
        if min_file_size * num_files > uncompressed_size:
            raise AssertionError(
                '`uncompressed_size` is smaller than the calculated minimum required size',
            )
        if compression in ['bzip2', 'bz2']:
            compression = zipfile.ZIP_BZIP2
        elif compression in ['lzma', 'xz']:
            compression = zipfile.ZIP_LZMA
        elif compression in ['deflate', 'gzip', 'gz']:
            compression = zipfile.ZIP_DEFLATED
        else:
            compression = zipfile.ZIP_STORED

        zip_buffer = io.BytesIO()
        remaining_size = uncompressed_size
        with zipfile.ZipFile(zip_buffer, mode='w', compression=compression) as zip_handle:
            for file_number in range(1, num_files + 1):
                filename = self.generator.pystr() + str(file_number)

                max_allowed_size = remaining_size - (num_files - file_number) * min_file_size
                if file_number < num_files:
                    file_size = self.generator.random.randint(min_file_size, max_allowed_size)
                    remaining_size = remaining_size - file_size
                else:
                    file_size = remaining_size

                data = self.generator.binary(file_size)
                zip_handle.writestr(filename, data)
        return zip_buffer.getvalue()

    def tar(self, uncompressed_size=65536, num_files=1, min_file_size=4096, compression=None):
        """Generate a bytes object containing a random valid tar file.

        The number and sizes of files contained inside the resulting archive can be controlled
        using the following arguments:

        - ``uncompressed_size`` - the total size of files before compression, 16 KiB by default
        - ``num_files`` - the number of files archived in resulting zip file, 1 by default
        - ``min_file_size`` - the minimum size of each file before compression, 4 KiB by default

        No compression is used by default, but setting ``compression`` to one of the values listed
        below will use the corresponding compression type.

        - ``'bzip2'`` or ``'bz2'`` for BZIP2
        - ``'lzma'`` or ``'xz'`` for LZMA
        - ``'gzip'`` or ``'gz'`` for GZIP

        :sample: uncompressed_size=256, num_files=4, min_file_size=32
        :sample: uncompressed_size=256, num_files=32, min_file_size=4, compression='bz2'
        """
        if any([
            not isinstance(num_files, int) or num_files <= 0,
            not isinstance(min_file_size, int) or min_file_size <= 0,
            not isinstance(uncompressed_size, int) or uncompressed_size <= 0,
        ]):
            raise ValueError(
                '`num_files`, `min_file_size`, and `uncompressed_size` must be positive integers',
            )
        if min_file_size * num_files > uncompressed_size:
            raise AssertionError(
                '`uncompressed_size` is smaller than the calculated minimum required size',
            )
        if compression in ['gzip', 'gz']:
            mode = 'w:gz'
        elif compression in ['bzip2', 'bz2']:
            mode = 'w:bz2'
        elif compression in ['lzma', 'xz']:
            mode = 'w:xz'
        else:
            mode = 'w'

        tar_buffer = io.BytesIO()
        remaining_size = uncompressed_size
        with tarfile.open(mode=mode, fileobj=tar_buffer) as tar_handle:
            for file_number in range(1, num_files + 1):
                file_buffer = io.BytesIO()
                filename = self.generator.pystr() + str(file_number)

                max_allowed_size = remaining_size - (num_files - file_number) * min_file_size
                if file_number < num_files:
                    file_size = self.generator.random.randint(min_file_size, max_allowed_size)
                    remaining_size = remaining_size - file_size
                else:
                    file_size = remaining_size

                tarinfo = tarfile.TarInfo(name=filename)
                data = self.generator.binary(file_size)
                file_buffer.write(data)
                tarinfo.size = len(file_buffer.getvalue())
                file_buffer.seek(0)
                tar_handle.addfile(tarinfo, file_buffer)
                file_buffer.close()
        return tar_buffer.getvalue()

    def dsv(self, dialect='faker-csv', header=None,
            data_columns=('{{name}}', '{{address}}'),
            num_rows=10, include_row_ids=False, **fmtparams):
        """Generate random delimiter-separated values.

        This method's behavior share some similarities with ``csv.writer``. The ``dialect`` and
        ``**fmtparams`` arguments are the same arguments expected by ``csv.writer`` to control its
        behavior, and instead of expecting a file-like object to where output will be written, the
        output is controlled by additional keyword arguments and is returned as a string.

        The ``dialect`` argument defaults to ``'faker-csv'`` which is the name of a ``csv.excel``
        subclass with full quoting enabled.

        The ``header`` argument expects a list or a tuple of strings that will serve as the header row
        if supplied. The ``data_columns`` argument expects a list or a tuple of string tokens, and these
        string tokens will be passed to  :meth:`pystr_format() <faker.providers.python.Provider.pystr_format>`
        for data generation. Both ``header`` and ``data_columns`` myst be of the same length.

        The ``num_rows`` argument controls how many rows of data to generate, and the ``include_row_ids``
        argument may be set to ``True`` to include a sequential row ID column.

        :sample: dialect='excel', data_columns=('{{name}}', '{{address}}')
        :sample: dialect='excel-tab', data_columns=('{{name}}', '{{address}}'), include_row_ids=True
        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=5, delimiter='$'
        """

        if not isinstance(num_rows, int) or num_rows <= 0:
            raise ValueError('`num_rows` must be a positive integer')
        if not isinstance(data_columns, (list, tuple)):
            raise TypeError('`data_columns` must be a tuple or a list')
        if header is not None:
            if not isinstance(header, (list, tuple)):
                raise TypeError('`header` must be a tuple or a list')
            if len(header) != len(data_columns):
                raise ValueError('`header` and `data_columns` must have matching lengths')

        dsv_buffer = io.StringIO()
        writer = csv.writer(dsv_buffer, dialect=dialect, **fmtparams)

        if header:
            if include_row_ids:
                header = list(header)
                header.insert(0, 'ID')
            writer.writerow(header)

        for row_num in range(1, num_rows + 1):
            row = [self.generator.pystr_format(column) for column in data_columns]
            if include_row_ids:
                row.insert(0, str(row_num))

            writer.writerow(row)

        return dsv_buffer.getvalue()

    def csv(self, header=None, data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False):
        """Generate random comma-separated values.

        For more information on the different arguments of this method, please refer to
        :meth:`dsv() <faker.providers.misc.Provider.dsv>` which is used under the hood.

        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False
        :sample: header=('Name', 'Address', 'Favorite Color'),
                data_columns=('{{name}}', '{{address}}', '{{safe_color_name}}'),
                num_rows=10, include_row_ids=True
        """
        return self.dsv(
            header=header, data_columns=data_columns, num_rows=num_rows,
            include_row_ids=include_row_ids, delimiter=',',
        )

    def tsv(self, header=None, data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False):
        """Generate random tab-separated values.

        For more information on the different arguments of this method, please refer to
        :meth:`dsv() <faker.providers.misc.Provider.dsv>` which is used under the hood.

        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False
        :sample: header=('Name', 'Address', 'Favorite Color'),
                data_columns=('{{name}}', '{{address}}', '{{safe_color_name}}'),
                num_rows=10, include_row_ids=True
        """
        return self.dsv(
            header=header, data_columns=data_columns, num_rows=num_rows,
            include_row_ids=include_row_ids, delimiter='\t',
        )

    def psv(self, header=None, data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False):
        """Generate random pipe-separated values.

        For more information on the different arguments of this method, please refer to
        :meth:`dsv() <faker.providers.misc.Provider.dsv>` which is used under the hood.

        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False
        :sample: header=('Name', 'Address', 'Favorite Color'),
                data_columns=('{{name}}', '{{address}}', '{{safe_color_name}}'),
                num_rows=10, include_row_ids=True
        """
        return self.dsv(
            header=header, data_columns=data_columns, num_rows=num_rows,
            include_row_ids=include_row_ids, delimiter='|',
        )



from .. import Provider as BaseProvider


# China jobs from
# https://js.51jobcdn.com/in/js/2016/layer/funtype_array_c.js?20180611
class Provider(BaseProvider):
    jobs = [
        "计算机硬件",
        "高级硬件工程师",
        "硬件工程师",
        "其他",
        "计算机软件",
        "高级软件工程师",
        "软件工程师",
        "软件UI设计师/工程师",
        "算法工程师",
        "仿真应用工程师",
        "ERP实施顾问",
        "ERP技术开发",
        "需求工程师",
        "系统集成工程师",
        "系统分析员",
        "系统工程师",
        "系统架构设计师",
        "数据库工程师/管理员",
        "计算机辅助设计工程师",
        "其他",
        "互联网/电子商务/网游",
        "互联网软件开发工程师",
        "语音/视频/图形开发工程师",
        "多媒体/游戏开发工程师",
        "手机应用开发工程师",
        "网站运营总监",
        "网站运营经理/主管",
        "网站运营专员",
        "产品总监",
        "产品经理/主管",
        "产品专员",
        "SEO/SEM",
        "网络推广总监",
        "网络推广经理/主管",
        "网络推广专员",
        "新媒体运营",
        "电子商务总监",
        "电子商务经理/主管",
        "电子商务专员",
        "网络工程师",
        "UI设计师/顾问",
        "用户体验（UE/UX）设计师",
        "大数据开发/分析",
        "Web前端开发",
        "网站架构设计师",
        "网站维护工程师",
        "系统管理员/网络管理员",
        "网站策划",
        "网站编辑",
        "网页设计/制作/美工",
        "脚本开发工程师",
        "游戏策划师",
        "游戏界面设计师",
        "Flash设计/开发",
        "特效设计师",
        "视觉设计师",
        "音效设计师",
        "网络信息安全工程师",
        "其他",
        "IT-管理",
        "首席技术执行官CTO/首席信息官CIO",
        "技术总监/经理",
        "信息技术经理/主管",
        "信息技术专员",
        "项目总监",
        "项目经理",
        "项目主管",
        "项目执行/协调人员",
        "其他",
        "IT-品管、技术支持及其它",
        "技术支持/维护经理",
        "技术支持/维护工程师",
        "网络管理(Helpdesk)",
        "网络维修",
        "计量工程师",
        "标准化工程师",
        "品质经理",
        "系统测试",
        "软件测试",
        "硬件测试",
        "测试员",
        "文档工程师",
        "配置管理工程师",
        "技术文员/助理",
        "手机维修",
        "电脑维修",
        "其他",
        "通信技术开发及应用",
        "通信技术工程师",
        "有线传输工程师",
        "无线通信工程师",
        "电信交换工程师",
        "数据通信工程师",
        "移动通信工程师",
        "电信网络工程师",
        "通信电源工程师",
        "增值产品开发工程师",
        "手机软件开发工程师",
        "其他",
        "电子/电器/半导体/仪器仪表",
        "集成电路IC设计/应用工程师",
        "IC验证工程师",
        "电子工程师/技术员",
        "电子技术研发工程师",
        "射频工程师",
        "电子/电器维修工程师/技师",
        "变压器与磁电工程师",
        "版图设计工程师",
        "电气工程师/技术员",
        "电路工程师/技术员(模拟/数字)",
        "电声/音响工程师/技术员",
        "激光/光电子技术",
        "半导体技术",
        "自动控制工程师/技术员",
        "电子软件开发(ARM/MCU...)",
        "嵌入式软件开发(Linux/单片机/PLC/DSP…)",
        "嵌入式硬件开发(主板机…)",
        "电池/电源开发",
        "FAE 现场应用工程师",
        "工艺工程师",
        "家用电器/数码产品研发",
        "仪器/仪表/计量分析师",
        "测试工程师",
        "安防系统工程师",
        "其他",
        "销售管理",
        "销售总监",
        "销售经理",
        "销售主管",
        "业务拓展主管/经理",
        "渠道/分销总监",
        "渠道/分销经理",
        "渠道/分销主管",
        "大客户管理",
        "客户经理/主管",
        "区域销售总监",
        "区域销售经理",
        "团购经理/主管",
        "其他",
        "销售人员",
        "大客户销售",
        "销售代表",
        "渠道/分销专员",
        "客户代表",
        "销售工程师",
        "电话销售",
        "网络/在线销售",
        "团购业务员",
        "经销商",
        "会籍顾问",
        "销售助理",
        "其他",
        "销售行政及商务",
        "销售行政经理/主管",
        "销售行政专员",
        "业务分析经理/主管",
        "业务分析专员/助理",
        "商务经理",
        "商务主管/专员",
        "商务助理",
        "销售行政助理",
        "其他",
        "客服及支持",
        "客服总监",
        "客服经理",
        "客服主管",
        "客服专员/助理",
        "客户关系经理/主管",
        "售前/售后技术支持经理",
        "售前/售后技术支持主管",
        "售前/售后技术支持工程师",
        "咨询热线/呼叫中心服务人员",
        "网络/在线客服",
        "投诉专员",
        "VIP专员",
        "其他",
        "财务/审计/税务",
        "首席财务官 CFO",
        "财务总监",
        "财务经理",
        "财务顾问",
        "财务主管/总账主管",
        "会计经理/会计主管",
        "会计",
        "出纳员",
        "财务助理/文员",
        "固定资产会计",
        "财务分析经理/主管",
        "财务分析员",
        "成本经理/成本主管",
        "成本管理员",
        "资金经理/主管",
        "资金专员",
        "审计经理/主管",
        "审计专员/助理",
        "税务经理/税务主管",
        "税务专员/助理",
        "统计员",
        "其他",
        "金融/证券/期货/投资",
        "证券/期货/外汇经纪人",
        "证券分析师",
        "股票/期货操盘手",
        "金融/经济研究员",
        "金融产品经理",
        "金融产品销售",
        "投资/基金项目经理",
        "投资/理财顾问",
        "投资银行业务",
        "投资银行财务分析",
        "融资经理/融资主管",
        "融资专员",
        "风险管理/控制",
        "拍卖/担保/典当业务",
        "其他",
        "银行",
        "行长/副行长",
        "银行客户总监",
        "个人业务部门经理/主管",
        "个人业务客户经理",
        "公司业务部门经理/主管",
        "公司业务客户经理",
        "综合业务经理/主管",
        "综合业务专员",
        "资产评估/分析",
        "风险控制",
        "信贷管理",
        "信审核查",
        "进出口/信用证结算",
        "外汇交易",
        "清算人员",
        "高级客户经理/客户经理",
        "客户主管/专员",
        "营业部大堂经理",
        "信用卡销售",
        "呼叫中心客服",
        "银行柜员",
        "其他",
        "保险",
        "保险精算师",
        "保险产品开发/项目策划",
        "保险业务经理/主管",
        "保险经纪人/保险代理",
        "理财顾问/财务规划师",
        "储备经理人",
        "保险电销",
        "保险核保",
        "保险理赔",
        "保险客户服务/续期管理",
        "保险培训师",
        "保险内勤",
        "契约管理",
        "其他",
        "生产/营运",
        "工厂经理/厂长",
        "总工程师/副总工程师",
        "项目总监",
        "项目经理/主管",
        "项目工程师",
        "营运经理",
        "营运主管",
        "生产总监",
        "生产经理/车间主任",
        "生产主管",
        "生产领班/组长",
        "生产计划/物料管理(PMC)",
        "生产文员",
        "设备主管",
        "化验员",
        "其他",
        "质量安全",
        "质量管理/测试经理(QA/QC经理)",
        "质量管理/测试主管(QA/QC主管)",
        "质量管理/测试工程师(QA/QC工程师)",
        "质量检验员/测试员",
        "可靠度工程师",
        "故障分析工程师",
        "认证工程师",
        "体系工程师",
        "审核员",
        "环境/健康/安全经理/主管（EHS）",
        "环境/健康/安全工程师（EHS）",
        "安全员",
        "供应商管理",
        "采购材料、设备质量管理",
        "其他",
        "工程/机械/能源",
        "技术研发经理/主管",
        "技术研发工程师",
        "产品工艺/制程工程师",
        "产品规划工程师",
        "项目管理",
        "实验室负责人/工程师",
        "工程/设备经理",
        "工程/设备主管",
        "工程/设备工程师",
        "工程/机械绘图员",
        "工业工程师",
        "材料工程师",
        "机械工程师",
        "结构工程师",
        "模具工程师",
        "机电工程师",
        "维修经理/主管",
        "维修工程师",
        "装配工程师/技师",
        "铸造/锻造工程师/技师",
        "注塑工程师/技师",
        "焊接工程师/技师",
        "夹具工程师/技师",
        "CNC工程师",
        "冲压工程师/技师",
        "锅炉工程师/技师",
        "电力工程师/技术员",
        "光源与照明工程",
        "光伏系统工程师",
        "汽车/摩托车工程师",
        "船舶工程师",
        "轨道交通工程师/技术员",
        "飞机维修机械师",
        "飞行器设计与制造",
        "水利/水电工程师",
        "空调/热能工程师",
        "石油天然气技术人员",
        "矿产勘探/地质勘测工程师",
        "其他",
        "汽车制造",
        "研发总监/部长/专家",
        "汽车机构工程师",
        "汽车设计工程师",
        "汽车电子工程师",
        "发动机/总装工程师",
        "动力总成工程师",
        "底盘工程师",
        "汽车项目管理",
        "汽车质量管理",
        "汽车安全性能工程师",
        "汽车装配工艺工程师",
        "电气/电器工程师",
        "附件系统工程师",
        "内外饰工程师",
        "车身/造型设计",
        "车辆质量工程师",
        "新能源电池工程师",
        "新能源电控工程师",
        "新能源电机工程师",
        "其他",
        "汽车销售与服务",
        "4S店经理/维修站经理",
        "汽车销售/经纪人",
        "汽车修理工",
        "汽车电工",
        "汽车钣金",
        "汽车喷漆",
        "汽车检验/检测",
        "汽车装饰美容",
        "洗车工",
        "售后服务/客户服务",
        "二手车评估师",
        "加油站工作员",
        "其他",
        "技工普工",
        "普工/操作工",
        "技工",
        "叉车/铲车工",
        "组装工",
        "包装工",
        "焊工",
        "氩弧焊工",
        "电工",
        "电力线路工",
        "旋压工",
        "仪表工",
        "电镀工",
        "喷塑工",
        "水工",
        "木工",
        "漆工",
        "空调工",
        "电梯工",
        "锅炉工",
        "学徒工",
        "其他",
        "服装/纺织/皮革",
        "服装/纺织设计总监",
        "服装/纺织设计",
        "服装/纺织/皮革工艺师",
        "面料辅料开发",
        "面料辅料采购",
        "服装/纺织/皮革跟单",
        "服装领班",
        "质量管理/验货员(QA/QC)",
        "板房/楦头/底格出格师",
        "电脑放码员",
        "纸样师/车板工",
        "裁床",
        "打样/制版",
        "裁剪工",
        "缝纫工",
        "手缝工",
        "烫工",
        "样衣工",
        "纺织工",
        "针织工",
        "配色工",
        "印染工",
        "漂染工",
        "挡车工",
        "整经工",
        "细纱工",
        "浆纱工",
        "其他",
        "采购",
        "采购总监",
        "采购经理",
        "采购主管",
        "采购员",
        "采购助理",
        "买手",
        "供应商开发",
        "其他",
        "贸易",
        "贸易/外贸经理/主管",
        "贸易/外贸专员/助理",
        "国内贸易人员",
        "业务跟单经理",
        "高级业务跟单",
        "业务跟单",
        "助理业务跟单",
        "其他",
        "物流/仓储",
        "物流总监",
        "物流经理",
        "物流主管",
        "物流专员/助理",
        "供应链总监",
        "供应链经理",
        "供应链主管/专员",
        "物料经理",
        "物料主管/专员",
        "仓库经理/主管",
        "仓库管理员",
        "订单处理员",
        "运输经理/主管",
        "项目经理/主管",
        "货运代理",
        "集装箱业务",
        "海关事务管理",
        "报关与报检",
        "单证员",
        "船务/空运陆运操作",
        "快递员",
        "调度员",
        "安检员",
        "理货员",
        "搬运工",
        "其他",
        "生物/制药/医疗器械",
        "生物工程/生物制药",
        "化学分析测试员",
        "医药技术研发管理人员",
        "医药技术研发人员",
        "医药学术推广",
        "临床研究员",
        "临床协调员",
        "临床数据分析员",
        "药品注册",
        "药品生产/质量管理",
        "药品市场推广经理",
        "药品市场推广主管/专员",
        "医药招商",
        "政府事务管理",
        "招投标管理",
        "医药销售经理/主管",
        "医药代表",
        "医药销售人员",
        "医疗器械注册",
        "医疗器械研发",
        "医疗器械生产/质量管理",
        "医疗器械市场推广",
        "医疗器械销售经理/主管",
        "医疗器械销售代表",
        "医疗器械维修人员",
        "其他",
        "化工",
        "化工技术应用/化工工程师",
        "化工实验室研究员/技术员",
        "涂料研发工程师",
        "配色技术员",
        "塑料工程师",
        "化妆品研发",
        "食品/饮料研发",
        "造纸研发",
        "其他",
        "医院/医疗/护理",
        "医院管理人员",
        "综合门诊/全科医生",
        "内科医生",
        "外科医生",
        "专科医生",
        "牙科医生",
        "美容整形师",
        "麻醉医生",
        "放射科医师",
        "理疗师",
        "中医科医生",
        "针灸/推拿",
        "儿科医生",
        "心理医生",
        "营养师",
        "药库主任/药剂师",
        "医药学检验",
        "公共卫生/疾病控制",
        "护理主任/护士长",
        "护士/护理人员",
        "兽医",
        "验光师",
        "其他",
        "广告",
        "广告客户总监/副总监",
        "广告客户经理",
        "广告客户主管/专员",
        "广告创意/设计经理",
        "广告创意总监",
        "广告创意/设计主管/专员",
        "广告制作执行",
        "美术指导",
        "文案/策划",
        "企业/业务发展经理",
        "企业策划人员",
        "其他",
        "公关/媒介",
        "公关总监",
        "公关经理",
        "公关主管",
        "公关专员",
        "会务/会展经理",
        "会务/会展主管",
        "会务/会展专员",
        "媒介经理",
        "媒介主管",
        "媒介专员",
        "公关/媒介助理",
        "媒介销售",
        "活动策划",
        "活动执行",
        "其他",
        "市场/营销",
        "市场/营销/拓展总监",
        "市场/营销/拓展经理",
        "市场/营销/拓展主管",
        "市场/营销/拓展专员",
        "市场助理",
        "市场分析/调研人员",
        "产品/品牌经理",
        "产品/品牌主管",
        "产品/品牌专员",
        "市场通路经理/主管",
        "市场通路专员",
        "市场企划经理/主管",
        "市场企划专员",
        "促销经理",
        "促销主管/督导",
        "促销员/导购",
        "选址拓展/新店开发",
        "其他",
        "影视/媒体",
        "影视策划/制作人员",
        "导演/编导",
        "艺术/设计总监",
        "艺术指导/舞台美术设计",
        "经纪人/星探",
        "主播/主持人",
        "摄影师/摄像师",
        "后期制作",
        "音效师",
        "配音员",
        "灯光师",
        "放映经理/主管",
        "放映员",
        "其他",
        "编辑出版",
        "总编/副总编",
        "编辑",
        "作家/撰稿人",
        "记者",
        "电话采编",
        "美术编辑",
        "排版设计",
        "出版/发行",
        "其他",
        "艺术/设计",
        "平面设计总监",
        "平面设计经理/主管",
        "平面设计师",
        "绘画",
        "动画/3D设计",
        "原画师",
        "展览/展示/店面设计",
        "多媒体设计",
        "包装设计",
        "工业/产品设计",
        "工艺品/珠宝设计鉴定",
        "家具/家居用品设计",
        "玩具设计",
        "其他",
        "建筑工程与装潢",
        "高级建筑工程师/总工",
        "建筑工程师",
        "建筑设计师",
        "市政工程师",
        "结构/土木/土建工程师",
        "公路/桥梁/港口/隧道工程",
        "岩土工程",
        "楼宇自动化",
        "建筑机电工程师",
        "智能大厦/综合布线/安防/弱电",
        "给排水/暖通工程",
        "幕墙工程师",
        "规划与设计",
        "室内设计",
        "园艺/园林/景观设计",
        "测绘/测量",
        "建筑制图/模型/渲染",
        "开发报建",
        "工程造价师/预结算经理",
        "预结算员",
        "建筑工程管理/项目经理",
        "建筑项目助理",
        "建筑工程验收",
        "工程监理",
        "合同管理",
        "安全员",
        "资料员",
        "建筑安装施工员",
        "砌筑工",
        "瓦工",
        "混凝土工",
        "浇注工",
        "钢筋工",
        "木工",
        "油漆工",
        "电梯工",
        "抹灰工",
        "施工开料工",
        "管道/暖通",
        "工长",
        "其他",
        "房地产开发",
        "房地产项目/策划经理",
        "房地产项目/策划主管/专员",
        "房地产投资管理",
        "房产项目配套工程师",
        "房地产项目招投标",
        "房地产投资分析",
        "房地产资产管理",
        "监察人员",
        "其他",
        "房地产销售与中介",
        "房地产销售经理/主管",
        "房地产销售",
        "房地产中介/置业顾问",
        "房地产评估",
        "房地产店长/经理",
        "房地产内勤",
        "房地产客服",
        "其他",
        "物业管理",
        "物业管理经理",
        "物业管理主管",
        "物业管理专员/助理",
        "前介工程师",
        "物业设施管理人员",
        "物业机电维修工",
        "物业维修员",
        "高级物业顾问/物业顾问",
        "物业招商/租赁/租售",
        "停车管理员",
        "保安经理",
        "保安人员",
        "物业机电工程师",
        "保洁",
        "绿化工",
        "其他",
        "人力资源",
        "人事总监",
        "人事经理",
        "人事主管",
        "人事专员",
        "人事助理",
        "招聘经理/主管",
        "招聘专员/助理",
        "薪资福利经理/主管",
        "薪资福利专员/助理",
        "绩效考核经理/主管",
        "绩效考核专员/助理",
        "培训经理/主管",
        "培训专员/助理/培训师",
        "企业文化/员工关系/工会管理",
        "人力资源信息系统专员",
        "其他",
        "高级管理",
        "首席执行官CEO/总裁/总经理",
        "首席运营官COO",
        "副总经理/副总裁",
        "合伙人",
        "总监/部门经理",
        "策略发展总监",
        "企业秘书/董事会秘书",
        "投资者关系",
        "办事处首席代表",
        "办事处/分公司/分支机构经理",
        "总裁助理/总经理助理",
        "其他",
        "行政/后勤",
        "行政总监",
        "行政经理/主管/办公室主任",
        "行政专员/助理",
        "经理助理/秘书",
        "党工团干事",
        "前台接待/总机/接待生",
        "图书管理员/资料管理员",
        "电脑操作员/打字员",
        "后勤",
        "其他",
        "咨询/顾问",
        "专业顾问",
        "咨询总监",
        "咨询经理",
        "专业培训师",
        "咨询员",
        "调研员",
        "猎头/人才中介",
        "情报信息分析人员",
        "其他",
        "律师/法务/合规",
        "律师/法律顾问",
        "律师助理",
        "法务经理",
        "法务主管/专员",
        "法务助理",
        "合规经理",
        "合规主管/专员",
        "知识产权/专利/商标",
        "其他",
        "教师",
        "校长",
        "大学教授",
        "讲师/助教",
        "中学教师",
        "小学教师",
        "幼教",
        "外语培训师",
        "院校教务管理人员",
        "兼职教师",
        "家教",
        "音乐/美术教师",
        "体育教师",
        "职业技术教师",
        "其他",
        "培训",
        "培训督导",
        "培训讲师",
        "培训策划",
        "培训产品开发",
        "培训/课程顾问",
        "培训助理",
        "其他",
        "科研",
        "科研管理人员",
        "科研人员",
        "餐饮服务",
        "店长/经理",
        "大堂经理",
        "餐厅领班",
        "餐饮服务员",
        "行政主厨/厨师长",
        "中餐厨师",
        "西餐厨师",
        "日式厨师",
        "面点师",
        "西点师",
        "厨师助理/学徒",
        "茶艺师",
        "咖啡师",
        "调酒师/侍酒师/吧台员",
        "礼仪/迎宾",
        "预订员",
        "收银员",
        "配菜/打荷",
        "传菜主管",
        "传菜员",
        "洗碗工",
        "送餐员",
        "杂工",
        "其他",
        "酒店旅游",
        "酒店/宾馆经理",
        "酒店/宾馆销售",
        "预定部主管",
        "预定员",
        "大堂经理",
        "酒店前台",
        "宴会管理",
        "宾客服务经理",
        "楼面经理",
        "客房服务员/楼面服务员",
        "健身房服务",
        "行李员",
        "管家部经理/主管",
        "清洁服务人员",
        "旅游产品销售",
        "行程管理/计调",
        "签证专员",
        "导游/旅行顾问",
        "票务",
        "机场代表",
        "其他",
        "美容保健",
        "美容店长",
        "美容培训师/导师",
        "美容顾问",
        "美容师",
        "美容助理",
        "彩妆培训师",
        "专柜彩妆顾问(BA)",
        "化妆师",
        "造型师",
        "美发店长",
        "发型师",
        "发型助理/学徒",
        "美甲师",
        "美体师",
        "瘦身顾问",
        "SPA 技师",
        "按摩",
        "足疗",
        "宠物护理/美容",
        "其他",
        "百货零售",
        "卖场经理/店长",
        "品类经理",
        "品牌/连锁招商管理",
        "奢侈品业务",
        "店员/营业员",
        "珠宝销售顾问",
        "督导/巡店",
        "导购员",
        "促销员",
        "收银主管",
        "收银员",
        "陈列员",
        "收货员",
        "理货员",
        "安防主管",
        "防损员/内保",
        "西点师/面包糕点加工",
        "生鲜食品加工/处理",
        "熟食加工",
        "兼职店员",
        "其他",
        "交通运输服务",
        "飞机机长/副机长",
        "空乘人员",
        "列车/地铁车长",
        "船长/副船长",
        "商务司机",
        "客运司机",
        "货运司机",
        "出租车司机",
        "班车司机",
        "列车/地铁司机",
        "特种车司机",
        "叉车司机",
        "铲车司机",
        "吊车司机",
        "驾校教练",
        "代驾",
        "地勤人员",
        "乘务员",
        "船员",
        "其他",
        "家政保洁",
        "家政服务/保姆",
        "月嫂",
        "育婴师/保育员",
        "护工",
        "清洁工",
        "钟点工",
        "洗衣工",
        "送水工",
        "保镖",
        "空调维修",
        "家电维修",
        "寻呼员/话务员",
        "其他",
        "公务员",
        "公务员",
        "翻译",
        "英语翻译",
        "日语翻译",
        "德语翻译",
        "法语翻译",
        "俄语翻译",
        "意大利语翻译",
        "西班牙语翻译",
        "葡萄牙语翻译",
        "阿拉伯语翻译",
        "韩语/朝鲜语翻译",
        "泰语翻译",
        "中国方言翻译",
        "其他语种翻译",
        "在校学生",
        "研究生",
        "大学/大专应届毕业生",
        "中专/职校生",
        "其他",
        "储备干部/培训生/实习生",
        "储备干部",
        "培训生",
        "实习生",
        "兼职",
        "兼职",
        "环保",
        "环保工程师",
        "环境影响评价工程师",
        "生态治理/规划",
        "环保检测",
        "水质检测员",
        "水处理工程师",
        "固废工程师",
        "废气处理工程师",
        "其它",
        "农/林/牧/渔",
        "养殖部主管",
        "场长(农/林/牧/渔业)",
        "农艺师",
        "畜牧师",
        "饲养员",
        "农业技术员",
        "动物营养/饲料研发",
        "其他",
        "网店淘宝",
        "网店/淘宝店长",
        "网店/淘宝运营",
        "网店店铺管理员",
        "网店/淘宝客服",
        "店铺推广",
        "网店美工",
        "网店模特",
        "其他",
        "机械机床",
        "数控操机",
        "数控编程",
        "机修工",
        "折弯工",
        "车工",
        "磨工",
        "铣工",
        "冲压工",
        "刨工",
        "钳工",
        "钻工",
        "镗工",
        "铆工",
        "钣金工",
        "抛光工",
        "切割技工",
        "模具工",
        "炼胶工",
        "硫化工",
        "吹膜工",
        "注塑工",
        "其他",
        "印刷包装",
        "印刷工",
        "校对/录入",
        "调色员",
        "烫金工",
        "晒版员",
        "印刷排版/制版",
        "装订工",
        "印刷机械机长",
        "数码直印/菲林输出",
        "调墨技师",
        "电分操作员",
        "打稿机操作员",
        "切纸机操作工",
        "裱胶工",
        "压痕工",
        "复卷工",
        "其他",
        "运动健身",
        "健身顾问/教练",
        "瑜伽老师",
        "舞蹈老师",
        "游泳教练",
        "救生员",
        "高尔夫教练",
        "体育运动教练",
        "其他",
        "休闲娱乐",
        "司仪",
        "婚礼/庆典策划服务",
        "DJ",
        "驻唱/歌手",
        "舞蹈演员",
        "模特",
        "演员/群众演员",
        "娱乐领班",
        "娱乐服务员",
        "前台迎宾",
        "其他",
        "其他",
        "驯兽师/助理驯兽师",
        "志愿者/社会工作者",
        "其他",
    ]



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{last_name}}, {{first_name}}',
    )

    first_names = (
        'Tomas', 'Lukas', 'Mantas', 'Deividas', 'Arnas', 'Artūras',
        'Karolis', 'Dovydas', 'Dominykas', 'Darius', 'Edvinas', 'Jonas',
        'Martynas', 'Kajus', 'Donatas', 'Andrius', 'Matas', 'Rokas',
        'Augustas', 'Danielius', 'Mindaugas', 'Paulius', 'Marius',
        'Armandas', 'Edgaras', 'Jokūbas', 'Nedas', 'Tadas', 'Nerijus',
        'Simonas', 'Vytautas', 'Artūras', 'Robertas', 'Eimantas', 'Arijus',
        'Nojus', 'Egidijus', 'Aurimas', 'Emilis', 'Laurynas', 'Edvardas',
        'Joris', 'Pijus', 'Erikas', 'Domas', 'Vilius', 'Evaldas', 'Justinas',
        'Aleksandras', 'Kristupas', 'Gabrielius', 'Benas', 'Gytis', 'Arminas',
        'Vakris', 'Tautvydas', 'Domantas', 'Justas', 'Markas', 'Antanas',
        'Arūnas', 'Ernestas', 'Aronas', 'Vaidas', 'Ąžuolas', 'Titas', 'Giedrius',
        'Ignas', 'Povilas', 'Saulius', 'Julius', 'Arvydas', 'Kęstutis', 'Rytis',
        'Aistis', 'Gediminas', 'Algirdas', 'Naglis', 'Irmantas', 'Rolandas',
        'Aivaras', 'Simas', 'Faustas', 'Ramūnas', 'Šarūnas', 'Gustas', 'Tajus',
        'Dainius', 'Arnoldas', 'Linas', 'Rojus', 'Adomas', 'Žygimantas',
        'Ričardas', 'Orestas', 'Kipras', 'Juozas', 'Audrius', 'Romualdas',
        'Petras', 'Eleonora', 'Raminta', 'Dovilė', 'Sandra', 'Dominyka', 'Ana',
        'Erika', 'Kristina', 'Gintarė', 'Rūta', 'Edita', 'Karina', 'Živilė',
        'Jolanta', 'Radvilė', 'Ramunė', 'Svetlana', 'Ugnė', 'Eglė', 'Viktorija',
        'Justina', 'Brigita', 'Rasa', 'Marija', 'Giedrė', 'Iveta', 'Sonata',
        'Vitalija', 'Adrija', 'Goda', 'Paulina', 'Kornelija', 'Liepa', 'Vakarė',
        'Milda', 'Meda', 'Vaida', 'Izabelė', 'Jovita', 'Irma', 'Žemyna', 'Leila',
        'Rimantė', 'Mantė', 'Rytė', 'Perla', 'Greta', 'Monika', 'Ieva', 'Indrė',
        'Ema', 'Aurelija', 'Smiltė', 'Ingrida', 'Simona', 'Amelija', 'Sigita',
        'Olivija', 'Laurita', 'Jorūnė', 'Leticija', 'Vigilija', 'Medėja', 'Laura',
        'Agnė', 'Evelina', 'Kotryna', 'Lėja', 'Aušra', 'Neringa', 'Gerda',
        'Jurgita', 'Rusnė', 'Aušrinė', 'Rita', 'Elena', 'Ineta', 'Ligita',
        'Vasarė', 'Vėjūnė', 'Ignė', 'Gytė', 'Ariana', 'Arielė', 'Vytė', 'Eidvilė',
        'Karolina', 'Miglė', 'Viltė', 'Jolanta', 'Enrika', 'Aurėja', 'Vanesa',
        'Darija', 'Reda', 'Milana', 'Rugilė', 'Diana',
    )

    last_names = (
        'Kazlauskas', 'Jankauskas', 'Petrauskas', 'Pocius', 'Stankevičius',
        'Vsiliauskas', 'Žukauskas', 'Butkus', 'Paulauskas', 'Urbonas',
        'Kavaliauskas', 'Sakalauskas', 'Žukauskas', 'Akelis', 'Ambrasas',
        'Kairys', 'Kalvaitis', 'Kalvelis', 'Kalvėnas', 'Kaupas', 'Kiška',
        'Gagys', 'Gailius', 'Gailys', 'Gaižauskas', 'Gaičiūnas', 'Galdikas',
        'Gintalas', 'Ginzburgas', 'Grinius', 'Gronskis', 'Nagys', 'Naujokas',
        'Narušis', 'Nausėda', 'Poška', 'Povilonis',
    )



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    """
    source: https://ja.wikipedia.org/wiki/%E8%81%B7%E6%A5%AD%E4%B8%80%E8%A6%A7
    """

    jobs = [
        'アイドル',
        'アーティスト',
        'アートディレクター',
        'アナウンサー',
        'アニメーター',
        '医師',
        'イラストレーター',
        '医療事務員',
        'ウェディングプランナー',
        'ウェブデザイナー',
        '占い師',
        '運転士',
        '映画監督',
        '営業',
        '栄養士',
        'エステティシャン',
        '絵本作家',
        '演歌歌手',
        'エンジニア' '演奏家',
        'お笑い芸人',
        '音楽家',
        '音響技術者',
        '介護ヘルパー',
        '気象予報士',
        '脚本家',
        '救急救命士',
        '行政書士',
        'グラフィックデザイナー',
        '経営者',
        '検察官',
        'ゲームクリエイター',
        '建築家',
        '航海士',
        'コピーライター',
        '高等学校教員',
        '公認会計士',
        '公務員',
        '裁判官',
        '作曲家',
        '歯科医師',
        '司法書士',
        '小説家',
        '寿司職人',
        '測量士',
        '大学教授',
        '調理師',
        '電気工事士',
        '農家',
        '配管工',
        'バスガイド',
        '花火師',
        '漫画家',
        'モデル',
        '薬剤師',
        'YouTuber',
        '和紙職人',
    ]



from ..de import Provider as AddressProvider


class Provider(AddressProvider):

    city_formats = ('{{city_name}}', )

    city_with_postcode_formats = ('{{postcode}} {{city}}', )

    street_name_formats = (
        '{{first_name}}-{{last_name}}-{{street_suffix_long}}',
        '{{last_name}}{{street_suffix_short}}',
    )
    street_address_formats = ('{{street_name}} {{building_number}}', )
    address_formats = ('{{street_address}}\n{{postcode}} {{city}}', )

    building_number_formats = ('###', '##', '#', '#/#')

    street_suffixes_long = (
        'Gasse', 'Platz', 'Ring', 'Straße', 'Weg', 'Allee',
    )
    street_suffixes_short = (
        'gasse', 'platz', 'ring', 'straße', 'str.', 'weg', 'allee',
    )

    postcode_formats = ('#####', )

    cities = (
        'Aachen', 'Ahaus', 'Altentreptow', 'Altötting', 'Amberg', 'Angermünde',
        'Anklam', 'Ansbach', 'Apolda', 'Arnstadt', 'Artern', 'Aschaffenburg',
        'Aue', 'Auerbach', 'Augsburg', 'Aurich', 'Backnang', 'Bad Brückenau',
        'Bad Freienwalde', 'Bad Kissingen', 'Bad Kreuznach', 'Bad Langensalza',
        'Bad Liebenwerda', 'Bad Mergentheim', 'Badalzungen', 'Badibling',
        'Badoberan', 'Bamberg', 'Bautzen', 'Bayreuth', 'Beeskow', 'Beilngries',
        'Belzig', 'Berchtesgaden', 'Bergzabern', 'Berlin', 'Bernburg',
        'Bersenbrück', 'Biedenkopf', 'Bischofswerda', 'Bitterfeld', 'Bogen',
        'Borken', 'Borna', 'Brand', 'Brandenburg', 'Bremen', 'Bremervörde',
        'Brilon', 'Bruchsal', 'Burg', 'Burgdorf', 'Burglengenfeld',
        'Böblingen', 'Büsingenm Hochrhein', 'Bützow', 'Calau', 'Calw', 'Celle',
        'Chemnitz', 'Cloppenburg', 'Coburg', 'Cottbus', 'Crailsheim',
        'Cuxhaven', 'Dachau', 'Darmstadt', 'Deggendorf', 'Delitzsch', 'Demmin',
        'Dessau', 'Dieburg', 'Diepholz', 'Dinkelsbühl', 'Dinslaken',
        'Donaueschingen', 'Dresden', 'Duderstadt', 'Döbeln', 'Düren',
        'Ebermannstadt', 'Ebern', 'Ebersberg', 'Eberswalde', 'Eckernförde',
        'Eggenfelden', 'Eichstätt', 'Eichstätt', 'Eilenburg', 'Einbeck',
        'Eisenach', 'Eisenberg', 'Eisenhüttenstadt', 'Eisleben', 'Emmendingen',
        'Erbisdorf', 'Erding', 'Erfurt', 'Erkelenz', 'Euskirchen', 'Eutin',
        'Fallingbostel', 'Feuchtwangen', 'Finsterwalde', 'Flöha', 'Forchheim',
        'Forst', 'Freising', 'Freital', 'Freudenstadt', 'Fulda',
        'Fürstenfeldbruck', 'Fürstenwalde', 'Füssen', 'Gadebusch',
        'Gardelegen', 'Garmisch-Partenkirchen', 'Geithain', 'Geldern',
        'Gelnhausen', 'Genthin', 'Gera', 'Germersheim', 'Gerolzhofen',
        'Gießen', 'Gifhorn', 'Goslar', 'Gotha', 'Grafenau', 'Gransee',
        'Greifswald', 'Greiz', 'Grevenbroich', 'Grevesmühlen',
        'Griesbach Rottal', 'Grimma', 'Grimmen', 'Groß-Gerau', 'Großenhain',
        'Gräfenhainichen', 'Guben', 'Gunzenhausen', 'Göppingen', 'Görlitz',
        'Göttingen', 'Günzburg', 'Güstrow', 'Gütersloh', 'Hagenow',
        'Hainichen', 'Halberstadt', 'Haldensleben', 'Hamburg', 'Hammelburg',
        'Hannover', 'Hannoversch Münden', 'Hansestadttralsund', 'Havelberg',
        'Hechingen', 'Heiligenstadt', 'Heinsberg', 'Helmstedt', 'Herford',
        'Hersbruck', 'Herzberg', 'Hettstedt', 'Hildburghausen', 'Hildesheim',
        'Hofgeismar', 'Hohenmölsen', 'Hohenstein-Ernstthal', 'Holzminden',
        'Hoyerswerda', 'Husum', 'Höxter', 'Hünfeld', 'Illertissen', 'Ilmenau',
        'Ingolstadt', 'Iserlohn', 'Jena', 'Jessen', 'Jülich', 'Jüterbog',
        'Kaiserslautern', 'Kamenz', 'Karlsruhe', 'Kassel', 'Kehl', 'Kelheim',
        'Kemnath', 'Kitzingen', 'Kleve', 'Klötze', 'Koblenz', 'Konstanz',
        'Kronach', 'Kulmbach', 'Kusel', 'Kyritz', 'Königs Wusterhausen',
        'Kötzting', 'Leipziger Land', 'Lemgo', 'Lichtenfels', 'Lippstadt',
        'Lobenstein', 'Luckau', 'Luckenwalde', 'Ludwigsburg', 'Ludwigslust',
        'Lörrach', 'Lübben', 'Lübeck', 'Lübz', 'Lüdenscheid', 'Lüdinghausen',
        'Lüneburg', 'Magdeburg', 'Main-Höchst', 'Mainburg', 'Malchin',
        'Mallersdorf', 'Marienberg', 'Marktheidenfeld', 'Mayen', 'Meiningen',
        'Meißen', 'Melle', 'Mellrichstadt', 'Melsungen', 'Meppen', 'Merseburg',
        'Mettmann', 'Miesbach', 'Miltenberg', 'Mittweida', 'Moers', 'Monschau',
        'Mühldorfm Inn', 'Mühlhausen', 'München', 'Nabburg', 'Naila', 'Nauen',
        'Neu-Ulm', 'Neubrandenburg', 'Neunburg vorm Wald', 'Neuruppin',
        'Neuss', 'Neustadtm Rübenberge', 'Neustadtner Waldnaab', 'Neustrelitz',
        'Niesky', 'Norden', 'Nordhausen', 'Northeim', 'Nördlingen',
        'Nürtingen', 'Oberviechtach', 'Ochsenfurt', 'Olpe', 'Oranienburg',
        'Oschatz', 'Osterburg', 'Osterodem Harz', 'Paderborn', 'Parchim',
        'Parsberg', 'Pasewalk', 'Passau', 'Pegnitz', 'Peine', 'Perleberg',
        'Pfaffenhofenner Ilm', 'Pinneberg', 'Pirmasens', 'Plauen', 'Potsdam',
        'Prenzlau', 'Pritzwalk', 'Pößneck', 'Quedlinburg', 'Querfurt',
        'Rastatt', 'Rathenow', 'Ravensburg', 'Recklinghausen', 'Regen',
        'Regensburg', 'Rehau', 'Reutlingen', 'Ribnitz-Damgarten', 'Riesa',
        'Rochlitz', 'Rockenhausen', 'Roding', 'Rosenheim', 'Rostock', 'Roth',
        'Rothenburg oberauber', 'Rottweil', 'Rudolstadt', 'Saarbrücken',
        'Saarlouis', 'Sangerhausen', 'Sankt Goar', 'Sankt Goarshausen',
        'Saulgau', 'Scheinfeld', 'Schleiz', 'Schlüchtern', 'Schmölln',
        'Schongau', 'Schrobenhausen', 'Schwabmünchen', 'Schwandorf',
        'Schwarzenberg', 'Schweinfurt', 'Schwerin', 'Schwäbisch Gmünd',
        'Schwäbisch Hall', 'Sebnitz', 'Seelow', 'Senftenberg', 'Siegen',
        'Sigmaringen', 'Soest', 'Soltau', 'Soltau', 'Sondershausen',
        'Sonneberg', 'Spremberg', 'Stade', 'Stade', 'Stadtroda',
        'Stadtsteinach', 'Staffelstein', 'Starnberg', 'Staßfurt', 'Steinfurt',
        'Stendal', 'Sternberg', 'Stollberg', 'Strasburg', 'Strausberg',
        'Stuttgart', 'Suhl', 'Sulzbach-Rosenberg', 'Säckingen', 'Sömmerda',
        'Tecklenburg', 'Teterow', 'Tirschenreuth', 'Torgau', 'Tuttlingen',
        'Tübingen', 'Ueckermünde', 'Uelzen', 'Uffenheim', 'Vechta',
        'Viechtach', 'Viersen', 'Vilsbiburg', 'Vohenstrauß', 'Waldmünchen',
        'Wanzleben', 'Waren', 'Warendorf', 'Weimar', 'Weißenfels',
        'Weißwasser', 'Werdau', 'Wernigerode', 'Wertingen', 'Wesel', 'Wetzlar',
        'Wiedenbrück', 'Wismar', 'Wittenberg', 'Wittmund', 'Wittstock',
        'Witzenhausen', 'Wolfach', 'Wolfenbüttel', 'Wolfratshausen', 'Wolgast',
        'Wolmirstedt', 'Worbis', 'Wunsiedel', 'Wurzen', 'Zerbst', 'Zeulenroda',
        'Zossen', 'Zschopau',
    )

    states = (
        'Baden-Württemberg', 'Bayern', 'Berlin', 'Brandenburg', 'Bremen',
        'Hamburg', 'Hessen', 'Mecklenburg-Vorpommern', 'Niedersachsen',
        'Nordrhein-Westfalen', 'Rheinland-Pfalz', 'Saarland', 'Sachsen',
        'Sachsen-Anhalt', 'Schleswig-Holstein', 'Thüringen',
    )

    def street_suffix_short(self):
        return self.random_element(self.street_suffixes_short)

    def street_suffix_long(self):
        return self.random_element(self.street_suffixes_long)

    def city_name(self):
        return self.random_element(self.cities)

    def state(self):
        return self.random_element(self.states)

    def city_with_postcode(self):
        pattern = self.random_element(self.city_with_postcode_formats)
        return self.generator.parse(pattern)



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = (
        '{{first_name_female}} {{last_name}}',
        '{{prefix_female}} {{first_name_female}} {{last_name}}',
    )

    formats_male = (
        '{{first_name_male}} {{last_name}}',
        '{{prefix_male}} {{first_name_male}} {{last_name}}',
    )

    formats = formats_male + formats_female

    first_names_female = (
        'اصيل', 'آلاء', 'آيات', 'ايمان', 'بهجة', 'تمام', 'بشري', 'حياة',
        'خاشعة', 'دانية', 'دعاء', 'زكية', 'نغم', 'لارا', 'زهرة', 'سبأ',
        'ضحى', 'ضياء', 'عالية', 'مريم', 'فداء', 'فرات', 'فردوس',
        'كاملة', 'كوثر', 'هاجر', 'هدى', 'يسرى', 'سجى', 'سلسبيل', 'شهد',
        'جنى', 'اسماءبناتمختلفةومعانيها:', 'ريتاج', 'يارا', 'وصاف',
        'ناردين', 'ميرا', 'مايا', 'مادلين', 'لينا', 'لورا', 'وسجايا',
        'روفيدا', 'ديمه', 'جيلان', 'جوانا', 'ألين', 'لتين', 'تالا',
        'سديم', 'جودي', 'ليان', 'دانة', 'ميار', 'لوجين', 'ربى', 'لورين',
        'ميرال', 'ريتال', 'جوليا', 'جالا', 'جوان', 'راما', 'هايدي',
        'ريفال', 'إلينا', 'أسيل', 'لوليا', 'ليساء', 'ميسون', 'جوين',
        'روبين', 'جمان', 'ميلاء', 'رواء', 'أناهيد', 'بيسان', 'ابتسام',
        'إباء', 'ابتكار', 'ابتهاج', 'ابتهال', 'بوران', 'محمد',
        'بنان', 'بيلسان', 'بتلاء', 'بدرالدّجى', 'تاليا', 'ترف', 'تالا',
        'ترانيم', 'جلنار', 'جميلة', 'جهراء', 'جواهر', 'جوريّة', 'ريمان',
        'ريما', 'ريناد', 'ريفال', 'راما', 'روعة', 'ريما', 'ريم',
        'اعتكاف', 'اعتماد', 'أغاريد', 'افتكار', 'أفراح', 'أفنان',
        'لاما', 'ليم', 'لوليا', 'ريمان', 'أجوان', 'بشرى', 'بلسم',
        'بلقيس', 'بلماء', 'بلند', 'بنان', 'بنفسج', 'بهابهاء', 'بهية',
        'نوره', 'نوف', 'نوال', 'ناديه', 'هدى', 'هناء', 'هيا', 'هند',
        'هنادي', 'وفاء', 'ياسمين', 'يسرى', 'غيداء', 'شادن', 'جود',
        'سلاف', 'جيلان', 'نشوة', 'ريان', 'دارين', '', 'أحلام', 'إخلاص',
        'أروى', 'أريج', 'أزهار', 'أسرار', 'آيات', 'ماذى', 'تولين',
        'هيام', 'ريناد', 'جميلة', 'حلا', 'عتاب', 'كرمة', 'ناهد', 'غوى',
        'ريف', 'بارعة', 'باسمة', 'باهرة', 'بتول', 'بثينة', 'أحمد',
    )

    first_names_male = (
        'تاج', 'تامر', 'تحسين', 'تقي', 'تمّام', 'تميم', 'توفيق', 'ترف',
        'تاج الدّين', 'تقيّ الدّين', 'ثائر', 'ثابت', 'ثامر', 'ثروت',
        'ثقيف', 'ثاقب', 'جابر', 'جاد', 'جاسم', 'جرير', 'جسور',
        'جعفر', 'جلاء', 'جلال', 'جليل', 'جمال', 'جميل', 'جدير',
        'جرّاح', 'جلال الدّين', 'جمال الدّين', 'جهاد', 'حاتم', 'حارث',
        'حازم', 'حافظ', 'حامد', 'حبّاب', 'حسام', 'حسن', 'حسيب',
        'حسين', 'حسني', 'حسنين', 'حقّي', 'حكيم', 'حليم', 'حمّاد',
        'حمدان', 'حمدي', 'حمزة', 'حمود', 'حميد', 'حنبل', 'حنفي',
        'حيّان', 'حيدر', 'حفيظ', 'خاطر', 'خافق', 'خالد', 'خالدي',
        'خلدون', 'خلف', 'خلوصي', 'خليفة', 'خليل', 'خميس', 'خيري',
        'خضر', 'خطيب', 'دؤوب', 'داني', 'داهي', 'داوود', 'دريد',
        'دليل', 'دهمان', 'ديسم', 'ذيب', 'ذكي', 'ذريع', 'رائد',
        'رائف', 'رابح', 'راتب', 'راجح', 'راجي', 'رازي', 'راشد', 'راضي',
        'راغب', 'رامز', 'رامي', 'رامح', 'راني', 'راوي', 'رؤوف', 'رباح',
        'ريّان', 'ربيع', 'رجاء', 'رجائي', 'رجب', 'رخاء', 'رستم',
        'رسمي', 'رشاد', 'رشدي', 'رشيد', 'رضوان', 'رفيق', 'رمحي',
        'رمزي', 'رمضان', 'رهيف', 'روحي', 'رافع', 'رئيس', 'رحيب',
        'رزين', 'راسم', 'رضي', 'زاخر', 'زاكي', 'زاهر', 'زاهي',
        'زايد', 'زبير', 'زغلول', 'زكريا', 'زكي', 'زهدي', 'زهران',
        'زهير', 'زياد', 'زيد', 'زيدان', 'زين', 'سائد', 'ساجد',
        'سخاء', 'ساجي', 'ساطع', 'سالم', 'سامح', 'ساهر', 'سامر',
        'سامي', 'ساهد', 'سراج', 'سرحان', 'سرور', 'سعد', 'سعدون',
        'سعدي', 'سعود', 'سعيد', 'سفيان', 'سفير', 'سلام', 'سلطان',
        'سلمان', 'سليمان', 'سموح', 'سمير', 'سنان', 'سنام', 'سهل',
        'سهوان', 'سهيل', 'سيّد', 'سليم', 'سراج الدّين', 'سيفالدّين',
        'شادي', 'شافع', 'شاكر', 'شامخ', 'شامل', 'شبلي', 'شبيب',
        'شدّاد', 'شريف', 'شعبان', 'شعيب', 'شفيع', 'شعلان', 'شكري',
        'شكيب', 'شهب', 'شهاب', 'شهم', 'شهير', 'شوقي', 'شجاع', 'شاطر',
        'شيّق', 'صائب', 'صابر', 'صاحب', 'صادح', 'صادق', 'صارم',
        'صافي', 'صالح', 'صامد', 'صباح', 'صبحي', 'صبري', 'صبور',
        'صبيح', 'صخر', 'صدقي', 'صدّام', 'صدّاح', 'صعب', 'صقر',
        'صلاح', 'صنديد', 'صهيب', 'صدر الدّين', 'صلاح الدّين', 'ضاحك',
        'ضاحي', 'ضحّاك', 'ضرغام', 'ضياء', 'ضيائي', 'ضياءالدّين',
        'طائع', 'طائف', 'طائل', 'طارق', 'طالب', 'طامح', 'طاهر',
        'طبّاع', 'طريف', 'طلال', 'طلعت', 'طموح', 'طه', 'طيّب',
        'طيّع', 'ظافر', 'ظبي', 'ظريف', 'ظهير', 'ظاعن', 'ظاهر',
        'عائد', 'عابد', 'عادل', 'عارف', 'عاصم', 'عاطف', 'عاقل',
        'عاكف', 'عالم', 'عامر', 'عبّاس', 'عبّود', 'عتريس', 'عتيد',
        'عربي', 'عثمان', 'عدلي', 'عدنان', 'عدوي', 'عرفات', 'عرفه',
        'عرفان', 'عزاز', 'عزّت', 'عزمي', 'عصام', 'عصمت', 'عطاء',
        'عفيف', 'عقيل', 'علاء', 'علّام', 'علوان', 'علي', 'عماد',
        'عمّار', 'عمر', 'عمران', 'عمرو', 'عمير', 'عاتب', 'عتيق',
        'عذب', 'عزيز', 'عبد الحقّ', 'عبدالله', 'عبدالرّحمن',
        'عزّالدّين', 'علاءالدّين', 'علم الدّين', 'عبد الإله', 'مُتعب',
        'عبد الباري', 'عبد الباقي', 'عبد التّواب', 'عبد الجبّار',
        'عبد الجليل', 'عبد الحفيظ', 'عبد الحكيم', 'عبد الحليم',
        'عبد الحيّ', 'عبد المحيي', 'عبد الخالق', 'عبد الرّزاق', 'هزار',
        'عبد الرّشيد', 'عبد الرّحمن', 'عبد الرّحيم', 'عبد الرّؤوف',
        'عبد السّميع', 'عبد الشّكور', 'عبد الصّمد', 'عبد العليم',
        'عبد الغفّار', 'عبد الغفور', 'عبد القادر', 'مهنّد', 'نزيه',
        'عبد القدّوس', 'عبد القهّار', 'عبد الكريم', 'عبد اللطيف',
        'عبد المجيد', 'عبد المولى', 'عبد العزيز', 'عبد السّلام',
        'عبد الملك', 'عبد الواحد', 'عبد الغني', 'غازي', 'غالب',
        'غالي', 'غانم', 'غزوان', 'غسّان', 'غطفان', 'غزير', 'غامد',
        'فائق', 'فاتح', 'فاخر', 'فادي', 'كامل', 'مصعب', 'ممتاز',
        'فرج', 'فارس', 'فارع', 'فاروق', 'فاضل', 'فالح', 'فايد', 'فتوح',
        'فتحي', 'فخر', 'فخري', 'فداء', 'فدائي', 'فراس', 'فرج', 'فرحان',
        'فرزدق', 'فضل', 'فطين', 'فكري', 'فلاح', 'فهد', 'فهمي', 'فؤاد',
        'فوّاز', 'فوزي', 'فضل', 'فيّاض', 'فيصل', 'فخر الدّين', 'قاسم',
        'قاصد', 'قانت', 'قائد', 'قحطان', 'قدري', 'قصي', 'قنوع', 'قيس',
        'قبس', 'قصيد', 'قطب', 'قطز', 'كارم', 'كاسر', 'كاشف', 'كاظم',
        'كايد', 'كافور', 'كتوم', 'كرم', 'كريم', 'كسّاب', 'كمال', 'كنار',
        'كنعان', 'كنان', 'كبير', 'كليم', 'لبيب', 'لبيد', 'لطفي', 'لطوف',
        'لفيف', 'لقمان', 'لقاء', 'لؤي', 'لهفان', 'ليث', 'لمّاح', 'مأمون',
        'ماجد', 'مازن', 'مالك', 'ماهر', 'محمّد', 'مُتوكّل', 'مُتولي',
        'مُتيّم', 'مجد', 'مجاهد', 'مَجدي', 'محجوب', 'محسن', 'محفوظ',
        'محمود', 'مختار', 'مخلص', 'مُخيمر', 'مدحت', 'مراد', 'مرادي',
        'مرتجي', 'مرتقي', 'مرزوق', 'مرسال', 'مرتضي', 'مُرسي', 'مرشد',
        'مُرضي', 'مرعي', 'مروان', 'مزهر', 'مسرور', 'مُسعف', 'مَسعد',
        'مسعود', 'مسلم', 'مشرف', 'مشرق', 'مشفق', 'مصباح', 'مصطفى',
        'مُصلح', 'مطاوع', 'مظهر', 'مُعتز', 'معتوق', 'معزّ', 'معمّر', 'معن',
        'معين', 'مفيد', 'مقداد', 'مقدام', 'مكّي', 'مكرّم', 'ملهم', 'يونس',
        'ممدوح', 'مُناضل', 'مناف', 'مُنذر', 'منيف', 'منتصر', 'مُنجد',
        'منسي', 'منصور', 'مُنير', 'منيب', 'منيع', 'مهدي', 'مهران',
        'مهيب', 'موسى', 'موفّق', 'مؤمن', 'مؤنس', 'مؤيّد', 'ميّاد', 'مياس',
        'ميسور', 'ميمون', 'ميثاق', 'معارف', 'محييالدّين', 'مشاري',
        'نائل', 'ناجح', 'ناجي', 'نادر', 'نادي', 'ناصر', 'ناضر', 'ناصيف',
        'ناظم', 'ناعم', 'نافذ', 'نافع', 'نبهان', 'نبيل', 'نبيه', 'نبراس',
        'نورالحقّ', 'نجدت', 'نجوان', 'نجيب', 'نديم', 'نذير', 'نزار',
        'نسيب', 'نشأت', 'نصر', 'نضال', 'نصري', 'نصور', 'نصوح', 'نظام',
        'نظمي', 'نعيم', 'نعمان', 'نمر', 'نوّاف', 'نوح', 'نوّار', 'نور',
        'نورس', 'نشوان', 'نوري', 'نيازي', 'ناصر الدّين', 'نصر الدّين',
        'نور الدّين', 'نجم الدّين', 'هادي', 'هاشم', 'هاني', 'هايل',
        'هلال', 'هلالي', 'همام', 'هيكل', 'هيمان', 'هيثم', 'وائل', 'واثق',
        'وادع', 'واصف', 'واصل', 'وثّاب', 'وجدي', 'وجيه', 'وحيد', 'ودود',
        'وديع', 'وريد', 'وسام', 'وسيم', 'وسيل', 'وصفي', 'وضّاح', 'وفائي',
        'وفيق', 'وليد', 'وليف', 'ياسر', 'يافع', 'ياقوت', 'يانع', 'يحيى',
        'يزيد', 'يسار', 'يسري', 'يعرب', 'يعقوب', 'يقين', 'يمام', 'يوسف',
    )

    first_names = first_names_male + first_names_female

    last_names = (
        'الخالدي', 'البديري', 'الشهابي', 'العفيفي', 'جزار',
        'الخطيب بني جماعة الكناني', 'الدجاني', 'الغوانمة', 'جار الله',
        'السروري', 'الامام', 'النقيب', 'المفتي', 'ابو السعود',
        'الفتياني', 'العلمي', 'بو مدين', 'نسيبة', 'النشاشيبي', 'العسلي',
        'الحسيني', 'الجاعوني', 'درويش', 'الأنصاري', 'جودة', 'النمري',
        'قطينة', 'الداودي', 'العارف', 'رصاص', 'البخاري', 'كمال',
        'الترجمان الصالح', 'غنيم', 'المؤقت', 'شتية', 'شرف', 'نور الدين',
        'الشعباني', 'الأيوبي', 'الجبشة', 'هندية', 'البشيتي', 'الوعري',
        'الموسوس', 'المظفر', 'الترهي', 'البغدادي', 'الهدمي', 'البامية',
        'الكلغاصي', 'اليوزباشي', 'المتولي', 'اسطمبولي', 'الألجاوي',
        'معتوق', 'حب رمان', 'القرجولي', 'نجم', 'طه', 'عبده', 'سموم',
        'نجيب', 'غوشة', 'اهرام', 'قرش', 'الكالوتي', 'حجازي', 'زحيكة',
        'جعفر', 'ازحيمان', 'الحواش', 'القضماني', 'طوطح', 'الشاويش',
        'بدرية', 'ابو الحاج', 'البيطار', 'صيام', 'قليبو', 'ارناؤوط',
        'الشرفاء', 'الحلاق', 'المملوك', 'السمان', 'طقش', 'وهبة',
        'عبد اللطيف', 'طزيز', 'السيفي', 'عويضة', 'القطب', 'الطحان',
        'النجار', 'القباني', 'عكاوي', 'الديسي', 'الزماميري', 'التوتنجي',
        'الحلواني', 'القزاز', 'الماني', 'الدقاق', 'الشامي', 'سوميرة',
        'ابو عيد', 'الخلفاوي', 'الدسوقي', 'المغربي', 'أفغاني', 'مراد',
        'زلاطيمو', 'سرندح', 'مشعشع', 'بحمدوني', 'بعلبكي', 'صيداوي',
        'صيداني', 'طرابلسي', 'جزيني', 'بيروتي', 'عرموني', 'متني',
        'شويفاتي', 'مزرعاني', 'بتروني', 'جبيلي', 'اميوني', 'زحلاوي',
        'الساحلي', 'القاعي', 'القلموني', 'البيسار القعقور', 'إياد',
        'الأزد', 'الأشراف', 'السادة', 'الأوس', 'أشجع', 'ألمع', 'أنمار',
        'بنو الأحمر', 'بنو الأحمر بن الحارث', 'بنو الأسمر', 'بنو أسد',
        'بنو أمية', 'أكلب', 'بنو النجار', 'البقوم', 'أولاد بوعزيز',
        'بارق', 'باهلة', 'بجيلة', 'بكر بن عبد مناة', 'بكر بن وائل',
        'بديرية', 'بلغازي', 'بلقرن', 'بلي', 'بيرقدار', 'بني بيات',
        'بكيل', 'ترابين', 'تغلب بن وائل', 'تميم', 'تنوخ', 'ثقيف',
        'الجعليين', 'جرهم', 'جديس', 'جذام', 'جهينة',
        'الحجر بن الهنوء بن الأزد', 'الحداء', 'الحكم بن سعد العشيرة',
        'بنو الحارث بن كعب', 'حرب', 'بنو حنيفة', 'حاشد', 'حميضة',
        'حمير', 'حوالة', 'الحويطات', 'الخزرج', 'بنو خالد', 'خثعم',
        'خزاعة', 'خندف', 'خولان', 'الدليم', 'الدواسر', 'بنو الدئل',
        'دوبلال', 'بنو ذي أصبح', 'راجح', 'بني رشيد', 'ربيعة', 'الرباب',
        'الرباطاب', 'السادة الراويون', 'الزرقان', 'زبيد', 'أولاد زيان',
        'بنو زيد', 'زهران', 'السهول', 'بنو سعد بن بكر',
        'بنو سعد بن ليث بن بكر', 'سليم', 'سبيع', 'الشايقية', 'الشحوح',
        'بنو شعبة', 'شمران', 'بنو شهر', 'بنو شيبان', 'بنو شيبة', 'شمر',
        'شهران', 'بنو صخر', 'بنو ضمرة', 'ضبيعة', 'طسم', 'طيء', 'الظفير',
        'عجرمة (العجارمة)', 'العجمان', 'العقيدات', 'العوازم', 'العوالق',
        'بنو العريج', 'عاملة', 'بنو عبس', 'بنو عجل', 'بنو عدي',
        'بنو عمرو', 'عامر بن صعصعة', 'عبد القيس', 'عتيبة', 'عدوان',
        'عذرة', 'عسير', 'عليان', 'عنز بن وائل', 'عنزة', 'عنس', 'عضل',
        'بني عطية', 'غامد', 'غطفان', 'بنو فراس', 'فراهيد', 'فهم',
        'القواسم', 'قحطان', 'قريش', 'قضاعة', 'قيس عيلان', 'بنو كنز',
        'الكواهلة', 'بنو كلب', 'كنانة', 'الكبابيش', 'كندة', 'كهلان',
        'الكثيري', 'بنو لام', 'لخم', 'بنو ليث', 'المرازيق', 'المنتفق',
        'الموركة', 'المهرة', 'بنو مالك', 'بنو معقل', 'بنو مهدي',
        'مزينة', 'مذحج', 'مرازيق البقوم', 'مضر', 'مطير', 'ميرفاب',
        'النمر', 'نهد', 'بني هاجر', 'بنو هاشم', 'بنو هلال',
        'قبيلة هذيل البقوم', 'هذيل', 'همدان', 'هوازن', 'بنو ياس',
        'بنو يعلى', 'يافع', 'يشكر',
    )

    prefixes_female = (
        'السيدة', 'الآنسة', 'الدكتورة', 'الأستاذة', 'المهندسة',
    )
    prefixes_male = ('السيد', 'المهندس', 'الدكتور', 'الأستاذ')



from collections import OrderedDict

from faker.providers.person.ru_RU import translit

from .. import CreditCard
from .. import Provider as CreditCardProvider


class Provider(CreditCardProvider):
    # Prefixes from: https://en.wikipedia.org/wiki/Payment_card_number#Issuer_identification_number_(IIN)
    prefix_visa = ['4']
    prefix_mastercard = ['51', '52', '53', '54', '55', '222%', '223', '224', '225', '226',
                         '227', '228', '229', '23', '24', '25', '26', '270', '271', '2720']
    prefix_mir = ['2200', '2201', '2202', '2203', '2204']
    prefix_maestro = ['50', '56', '57', '58', '59', '60', '61', '62', '63', '64', '65', '66', '67', '68', '69']
    prefix_amex = ['34', '37']
    prefix_unionpay = ['62', '81']

    credit_card_types = OrderedDict((
        ('visa', CreditCard('Visa', prefix_visa, security_code='CVV2')),
        ('mastercard', CreditCard('Mastercard', prefix_mastercard, security_code='CVC2')),
        ('mir', CreditCard('МИР', prefix_mir)),
        ('maestro', CreditCard('Maestro', prefix_maestro, security_code='CVV2')),
        ('amex', CreditCard('American Express', prefix_amex, 15, security_code='CID', security_code_length=4)),
        ('unionpay', CreditCard('Union Pay', prefix_unionpay)),
    ))

    def credit_card_expire(self, start='now', end='+4y', date_format='%m/%y'):
        expire_date = self.generator.date_time_between(start, end)
        return expire_date.strftime(date_format)

    def credit_card_full(self, card_type=None):
        card = self._credit_card_type(card_type)

        tpl = ('{provider}\n'
               '{owner}\n'
               '{number} {expire_date}\n'
               '{security}: {security_nb}\n'
               '{issuer}')

        tpl = tpl.format(provider=card.name,
                         owner=translit(
                             self.generator.parse(self.random_element(["{{first_name_male}} {{last_name_male}}",
                                                                       "{{first_name_female}} {{last_name_female}}"]))),
                         number=self.credit_card_number(card),
                         expire_date=self.credit_card_expire(),
                         security=card.security_code,
                         security_nb=self.credit_card_security_code(card),
                         issuer=self.generator.parse("{{bank}}"))

        return self.generator.parse(tpl)
